@@ -34,7 +34,7 @@
 #include <stdio.h>
 #include <string.h>
 #ifdef HAVE_STDLIB_H
-#include <stdlib.h> /* for abort() as used in errors */
+#include <stdlib.h>
 #endif
 
 #include <redland.h>
@@ -120,11 +120,8 @@ librdf_parser_register_factory(librdf_world *world,
 #endif
 
   if(!world->parsers) {
-#ifdef RAPTOR_V2_AVAILABLE
     world->parsers = raptor_new_sequence((raptor_data_free_handler)librdf_free_parser_factory, NULL);
-#else
-    world->parsers = raptor_new_sequence((raptor_sequence_free_handler *)librdf_free_parser_factory, NULL);
-#endif
+
     if(!world->parsers)
       goto oom;
   }
@@ -183,7 +180,7 @@ librdf_parser_register_factory(librdf_world *world,
 /**
  * librdf_get_parser_factory:
  * @world: redland world object
- * @name: the name of the factory (NULL or empty string if don't care)
+ * @name: the name of the factory (or NULL or empty string if don't care)
  * @mime_type: the MIME type of the syntax (NULL or empty if don't care)
  * @type_uri: URI of syntax (NULL if not used)
  *
@@ -266,6 +263,8 @@ librdf_get_parser_factory(librdf_world *world,
  *
  * Get information on parsers.
  * 
+ * @Deprecated: use librdf_parser_get_description() to return more information in a static structure.
+ *
  * Return value: non 0 on failure of if counter is out of range
  **/
 int
@@ -290,6 +289,26 @@ librdf_parser_enumerate(librdf_world* world,
 }
 
 
+/**
+ * librdf_parser_get_description:
+ * @world: world object
+ * @counter: index into the list of parsers
+ *
+ * Get parser descriptive syntax information
+ * 
+ * Return value: description or NULL if counter is out of range
+ **/
+const raptor_syntax_description*
+librdf_parser_get_description(librdf_world* world, 
+                              unsigned int counter)
+{
+  librdf_world_open(world);
+
+  return raptor_world_get_parser_description(world->raptor_world_ptr,
+                                             counter);
+}
+
+
 /**
  * librdf_parser_check_name:
  * @world: redland world object
@@ -323,7 +342,7 @@ librdf_parser_check_name(librdf_world* world, const char *name)
 /**
  * librdf_new_parser:
  * @world: redland world object
- * @name: the parser factory name
+ * @name: the parser factory name (or NULL or empty string if don't care)
  * @mime_type: the MIME type of the syntax (NULL if not used)
  * @type_uri: URI of syntax (NULL if not used)
  *
@@ -343,9 +362,23 @@ librdf_new_parser(librdf_world *world,
 
   librdf_world_open(world);
 
-  factory=librdf_get_parser_factory(world, name, mime_type, type_uri);
-  if(!factory)
+  factory = librdf_get_parser_factory(world, name, mime_type, type_uri);
+  if(!factory) {
+    if(name)
+      librdf_log(world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+                 "parser '%s' not found", name);
+    else if(mime_type)
+      librdf_log(world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+                 "parser for mime_type '%s' not found", mime_type);
+    else if(type_uri)
+      librdf_log(world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+                 "parser for type URI '%s' not found", 
+                 librdf_uri_as_string(type_uri));
+    else
+      librdf_log(world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+                 "default parser not found");
     return NULL;
+  }
 
   return librdf_new_parser_from_factory(world, factory);
 }
@@ -783,7 +816,7 @@ librdf_parser_guess_name2(librdf_world* world,
                           const unsigned char *identifier)
 {
   int len = buffer ? strlen((const char *)buffer) : 0;
-#ifdef RAPTOR_V2_AVAILABLE
+
   /* can do nothing if called with no world */
   if(!world || !world->raptor_world_ptr)
     return NULL;
@@ -792,11 +825,6 @@ librdf_parser_guess_name2(librdf_world* world,
                                         NULL, mime_type, buffer,
                                         len,
                                         identifier);
-#else
-  return raptor_guess_parser_name(NULL, mime_type, buffer,
-                                  len,
-                                  identifier);
-#endif
 }
 
 
