@@ -35,7 +35,7 @@
 #include <string.h>
 #include <sys/types.h>
 #ifdef HAVE_STDLIB_H
-#include <stdlib.h> /* for abort() as used in errors */
+#include <stdlib.h>
 #endif
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
@@ -68,32 +68,39 @@ static void rasqal_redland_free_triples_source(void *user_data);
 
 
 static void
-librdf_query_rasqal_error_handler(void *data, raptor_locator *locator,
-                                  const char *message) 
+librdf_query_rasqal_log_handler(void *data, raptor_log_message *message)
 {
-  librdf_query* query=(librdf_query*)data;
-  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
+  librdf_world *world = (librdf_world *)data;
+  librdf_log_level level;
 
-  context->errors++;
+  /* Map raptor2 fatal/error/warning log levels to librdf log levels,
+     ignore others */
+  switch(message->level) {
+    case RAPTOR_LOG_LEVEL_FATAL:
+      level = LIBRDF_LOG_FATAL;
+      break;
 
-  librdf_log_simple(query->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_QUERY, locator, message);
+    case RAPTOR_LOG_LEVEL_ERROR:
+      level = LIBRDF_LOG_ERROR;
+      break;
+
+    case RAPTOR_LOG_LEVEL_WARN:
+      level = LIBRDF_LOG_WARN;
+      break;
+
+    case RAPTOR_LOG_LEVEL_INFO:
+    case RAPTOR_LOG_LEVEL_DEBUG:
+    case RAPTOR_LOG_LEVEL_TRACE:
+    case RAPTOR_LOG_LEVEL_NONE:
+    default:
+      return;
+  }
+
+  librdf_log_simple(world, 0, level, LIBRDF_FROM_QUERY, message->locator,
+                    message->text);
 }
 
 
-static void
-librdf_query_rasqal_warning_handler(void *data, raptor_locator *locator,
-                                    const char *message) 
-{
-  librdf_query* query=(librdf_query*)data;
-  librdf_query_rasqal_context *context=(librdf_query_rasqal_context*)query->context;
-
-  context->warnings++;
-
-  librdf_log_simple(query->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_QUERY, locator, message);
-}
-
-
-
 /* functions implementing query api */
 
 
@@ -116,11 +123,8 @@ librdf_query_rasqal_init(librdf_query* query,
 
   rasqal_query_set_user_data(context->rq, query);
 
-  rasqal_query_set_error_handler(context->rq, query,
-                           librdf_query_rasqal_error_handler);
-  rasqal_query_set_warning_handler(context->rq, query,
-                             librdf_query_rasqal_warning_handler);
-
+  rasqal_world_set_log_handler(query->world->rasqal_world_ptr, query->world,
+                               librdf_query_rasqal_log_handler);
 
   len=strlen((const char*)query_string);
   query_string_copy=(unsigned char*)LIBRDF_MALLOC(cstring, len+1);
@@ -152,27 +156,54 @@ librdf_query_rasqal_terminate(librdf_query* query)
 }
 
 
+/*
+ * This function and RASQAL_LITERAL_UDT first appears in Rasqal 0.9.17
+ *
+ * FIXME: Remove this code when minimum rasqal version >= 0.9.17
+ */
+#ifndef RASQAL_LITERAL_UDT_DEFINED
+static rasqal_literal_type
+rasqal_literal_get_rdf_term_type(rasqal_literal* l)
+{
+  rasqal_literal_type type = l->type;
+  
+  /* squash literal datatypes into one type: RDF Literal */
+  if(type >= RASQAL_LITERAL_FIRST_XSD &&
+     type <= RASQAL_LITERAL_LAST_XSD)
+    type = RASQAL_LITERAL_STRING;
+
+  if(type != RASQAL_LITERAL_URI &&
+     type != RASQAL_LITERAL_STRING &&
+     type != RASQAL_LITERAL_BLANK)
+    type = RASQAL_LITERAL_UNKNOWN;
+
+  return type;
+}
+#endif
+
+
 static librdf_node*
 rasqal_literal_to_redland_node(librdf_world *world, rasqal_literal* l)
 {
+  rasqal_literal_type type;
+
   if(!l)
     return NULL;
   
-  if(l->type == RASQAL_LITERAL_URI)
+  type = rasqal_literal_get_rdf_term_type(l);
+
+  if(type == RASQAL_LITERAL_URI)
     return librdf_new_node_from_uri(world, (librdf_uri*)l->value.uri);
-  else if (l->type == RASQAL_LITERAL_STRING ||
-           l->type == RASQAL_LITERAL_INTEGER ||
-           l->type == RASQAL_LITERAL_DOUBLE ||
-           l->type == RASQAL_LITERAL_BOOLEAN)
+  else if (type == RASQAL_LITERAL_STRING)
     return librdf_new_node_from_typed_literal(world, 
                                               (unsigned char*)l->string, 
                                               l->language, 
                                               (librdf_uri*)l->datatype);
-  else if (l->type == RASQAL_LITERAL_BLANK)
+  else if (type == RASQAL_LITERAL_BLANK)
     return librdf_new_node_from_blank_identifier(world,
                                                  (unsigned char*)l->string);
   else {
-    LIBRDF_DEBUG2("Could not convert literal type %d to librdf_node", l->type);
+    LIBRDF_DEBUG2("Could not convert literal type %d to librdf_node", type);
     abort();
   }
 
@@ -180,14 +211,14 @@ rasqal_literal_to_redland_node(librdf_world *world, rasqal_literal* l)
 }
 
 
-static rasqal_literal*
-redland_node_to_rasqal_literal(librdf_node *node)
+rasqal_literal*
+redland_node_to_rasqal_literal(librdf_world* world, librdf_node *node)
 {
   rasqal_literal* l;
   
   if(librdf_node_is_resource(node)) {
     raptor_uri* uri=(raptor_uri*)librdf_new_uri_from_uri(librdf_node_get_uri(node));
-    l=rasqal_new_uri_literal(node->world->rasqal_world_ptr, uri); /* transfer uri ownership to literal */
+    l=rasqal_new_uri_literal(world->rasqal_world_ptr, uri); /* transfer uri ownership to literal */
   } else if(librdf_node_is_literal(node)) {
     unsigned char *string;
     librdf_uri *uri;
@@ -215,7 +246,7 @@ redland_node_to_rasqal_literal(librdf_node *node)
     if(uri)
       new_datatype=(raptor_uri*)librdf_new_uri_from_uri(uri);
     /* transfer new_string,new_language,new_datatype ownership to literal */
-    l=rasqal_new_string_literal(node->world->rasqal_world_ptr, (const unsigned char*)new_string, new_language, new_datatype, NULL);
+    l = rasqal_new_string_literal(world->rasqal_world_ptr, (const unsigned char*)new_string, new_language, new_datatype, NULL);
   } else {
     unsigned char *blank=librdf_node_get_blank_identifier(node);
     unsigned char *new_blank;
@@ -226,7 +257,7 @@ redland_node_to_rasqal_literal(librdf_node *node)
       return NULL;
     strcpy((char*)new_blank, (const char*)blank);
     /* transfer new_blank ownership to literal */
-    l=rasqal_new_simple_literal(node->world->rasqal_world_ptr, RASQAL_LITERAL_BLANK, (const unsigned char*)new_blank);
+    l = rasqal_new_simple_literal(world->rasqal_world_ptr, RASQAL_LITERAL_BLANK, (const unsigned char*)new_blank);
   }
 
   return l;
@@ -251,8 +282,14 @@ rasqal_redland_new_triples_source(rasqal_query* rdf_query,
   rasqal_redland_triples_source_user_data* rtsc=(rasqal_redland_triples_source_user_data*)user_data;
   raptor_sequence *seq;
   librdf_query_rasqal_context *context;
+  librdf_iterator* cit;
 
-  seq=rasqal_query_get_data_graph_sequence(rdf_query);
+  rtsc->world = world;
+  rtsc->query = (librdf_query*)rasqal_query_get_user_data(rdf_query);
+  context = (librdf_query_rasqal_context*)rtsc->query->context;
+  rtsc->model = context->model;
+
+  seq = rasqal_query_get_data_graph_sequence(rdf_query);
   
   /* FIXME: queries with data graphs in them (such as FROM in SPARQL)
    * are deleted, so that there are no unexpected data loads
@@ -264,11 +301,34 @@ rasqal_redland_new_triples_source(rasqal_query* rdf_query,
     }
   }
 
-  rtsc->world=world;
+  if(librdf_model_supports_contexts(rtsc->model)) {
+    /* Add all contexts (named graphs) to the query so Rasqal can bind them */
+    cit = librdf_model_get_contexts(rtsc->model);
+    while(!librdf_iterator_end(cit)) {
+      librdf_node* node = (librdf_node*)librdf_iterator_get_object(cit);
+      librdf_uri* uri;
+      raptor_uri* source_uri;
+      rasqal_data_graph* dg;
 
-  rtsc->query=(librdf_query*)rasqal_query_get_user_data(rdf_query);
-  context=(librdf_query_rasqal_context*)rtsc->query->context;
-  rtsc->model=context->model;
+      uri = librdf_node_get_uri(node);
+      source_uri = (raptor_uri*)raptor_new_uri(world->raptor_world_ptr,
+                                               librdf_uri_as_string(uri));
+
+      dg = rasqal_new_data_graph_from_uri(world->rasqal_world_ptr,
+                                          source_uri, source_uri,
+                                          RASQAL_DATA_GRAPH_NAMED,
+                                          NULL, NULL, NULL);
+      rasqal_query_add_data_graph(rdf_query, dg);
+
+      raptor_free_uri(source_uri);
+      librdf_iterator_next(cit);
+    }
+    librdf_free_iterator(cit);
+  }
+
+#ifdef RASQAL_TRIPLES_SOURCE_MIN_VERSION
+  rts->version = 1;
+#endif
 
   rts->init_triples_match=rasqal_redland_init_triples_match;
   rts->triple_present=rasqal_redland_triple_present;
@@ -309,11 +369,24 @@ rasqal_redland_free_triples_source(void *user_data)
 }
 
 
-static void
+#ifdef RASQAL_TRIPLES_SOURCE_FACTORY_MIN_VERSION
+#define RTS_RET int
+#else
+#define RTS_RET void
+#endif
+
+static RTS_RET
 rasqal_redland_register_triples_source_factory(rasqal_triples_source_factory *factory) 
 {
+#ifdef RASQAL_TRIPLES_SOURCE_FACTORY_MIN_VERSION
+  factory->version = 1;
+#endif
   factory->user_data_size=sizeof(rasqal_redland_triples_source_user_data);
   factory->new_triples_source=rasqal_redland_new_triples_source;
+
+#ifdef RASQAL_TRIPLES_SOURCE_FACTORY_MIN_VERSION
+  return 0;
+#endif
 }
 
 
@@ -336,12 +409,13 @@ rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
   rasqal_literal* l;
   librdf_statement* statement;
   rasqal_triple_parts result=(rasqal_triple_parts)0;
+  librdf_world *world = rtmc->stream->world;
 
   statement=librdf_stream_get_object(rtmc->stream);
   if(!statement)
     return (rasqal_triple_parts)0;
   
-#ifdef RASQAL_DEBUG
+#if LIBRDF_DEBUG > 1
   LIBRDF_DEBUG1("  matched statement ");
   librdf_statement_print(statement, stderr);
   fputc('\n', stderr);
@@ -350,8 +424,11 @@ rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
   /* set 1 or 2 variable values from the fields of statement */
 
   if(bindings[0] && (parts & RASQAL_TRIPLE_SUBJECT)) {
+#if LIBRDF_DEBUG > 1
     LIBRDF_DEBUG1("binding subject to variable\n");
-    l=redland_node_to_rasqal_literal(librdf_statement_get_subject(statement));
+#endif
+    l = redland_node_to_rasqal_literal(world,
+                                       librdf_statement_get_subject(statement));
     rasqal_variable_set_value(bindings[0], l);
     result= RASQAL_TRIPLE_SUBJECT;
   }
@@ -362,10 +439,15 @@ rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
       if(!librdf_node_equals(librdf_statement_get_subject(statement),
                              librdf_statement_get_predicate(statement)))
         return (rasqal_triple_parts)0;
+#if LIBRDF_DEBUG > 1
       LIBRDF_DEBUG1("subject and predicate values match\n");
+#endif
     } else {
+#if LIBRDF_DEBUG > 1
       LIBRDF_DEBUG1("binding predicate to variable\n");
-      l=redland_node_to_rasqal_literal(librdf_statement_get_predicate(statement));
+#endif
+      l = redland_node_to_rasqal_literal(world,
+                                         librdf_statement_get_predicate(statement));
       rasqal_variable_set_value(bindings[1], l);
       result= (rasqal_triple_parts)(result | RASQAL_TRIPLE_PREDICATE);
     }
@@ -380,7 +462,9 @@ rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
                              librdf_statement_get_object(statement)))
         return (rasqal_triple_parts)0;
       bind=0;
+#if LIBRDF_DEBUG > 1
       LIBRDF_DEBUG1("subject and object values match\n");
+#endif
     }
     if((bindings[1] == bindings[2]) && 
        !(bindings[0] == bindings[1])) {
@@ -392,12 +476,17 @@ rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
                              librdf_statement_get_object(statement)))
         return (rasqal_triple_parts)0;
       bind=0;
+#if LIBRDF_DEBUG > 1
       LIBRDF_DEBUG1("predicate and object values match\n");
+#endif
     }
     
     if(bind) {
+#if LIBRDF_DEBUG > 1
       LIBRDF_DEBUG1("binding object to variable\n");
-      l=redland_node_to_rasqal_literal(librdf_statement_get_object(statement));
+#endif
+      l = redland_node_to_rasqal_literal(world,
+                                         librdf_statement_get_object(statement));
       rasqal_variable_set_value(bindings[2], l);
       result= (rasqal_triple_parts)(result | RASQAL_TRIPLE_OBJECT);
     }
@@ -406,7 +495,7 @@ rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
   /* Contexts */
   if(bindings[3] && (parts & RASQAL_TRIPLE_ORIGIN)) {
     int bind=1;
-    librdf_node* context_node=(librdf_node*)librdf_stream_get_context(rtmc->stream);
+    librdf_node* context_node = librdf_stream_get_context2(rtmc->stream);
     
     if(bindings[0] == bindings[3]) {
       /* check matching (?x, ..., ...) in context ?x */
@@ -414,7 +503,9 @@ rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
                              context_node))
         return (rasqal_triple_parts)0;
       bind=0;
+#if LIBRDF_DEBUG > 1
       LIBRDF_DEBUG1("subject and context values match\n");
+#endif
     }
 
     if(bindings[1] == bindings[3]) {
@@ -423,7 +514,9 @@ rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
                              context_node))
         return (rasqal_triple_parts)0;
       bind=0;
+#if LIBRDF_DEBUG > 1
       LIBRDF_DEBUG1("predicate and context values match\n");
+#endif
     }
 
     if(bindings[2] == bindings[3]) {
@@ -432,13 +525,17 @@ rasqal_redland_bind_match(struct rasqal_triples_match_s* rtm,
                              context_node))
         return (rasqal_triple_parts)0;
       bind=0;
+#if LIBRDF_DEBUG > 1
       LIBRDF_DEBUG1("object and context values match\n");
+#endif
     }
 
     if(bind) {
+#if LIBRDF_DEBUG > 1
       LIBRDF_DEBUG1("binding origin to variable\n");
+#endif
       if(context_node)
-        l=redland_node_to_rasqal_literal(context_node);
+        l = redland_node_to_rasqal_literal(world, context_node);
       else
         l=NULL;
       rasqal_variable_set_value(bindings[3], l);
@@ -565,7 +662,7 @@ rasqal_redland_init_triples_match(rasqal_triples_match* rtm,
   if(!rtmc->qstatement)
     return 1;
 
-#ifdef RASQAL_DEBUG
+#if LIBDF_DEBUG > 1
   LIBRDF_DEBUG1("query statement: ");
   librdf_statement_print(rtmc->qstatement, stderr);
   if(rtmc->origin) {
@@ -585,7 +682,9 @@ rasqal_redland_init_triples_match(rasqal_triples_match* rtm,
   if(!rtmc->stream)
     return 1;
 
+#if LIBDF_DEBUG > 1
   LIBRDF_DEBUG1("rasqal_init_triples_match done\n");
+#endif
 
   return 0;
 }
@@ -890,16 +989,16 @@ librdf_query_rasqal_query_results_update_statement(void* context)
 
   /* subject */
   
-  if(rstatement->subject_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS) {
-    node=librdf_new_node_from_blank_identifier(world, (const unsigned char*)rstatement->subject);
-  } else if (rstatement->subject_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE) {
-    node=librdf_new_node_from_uri_string(world,
-                                         librdf_uri_as_string((librdf_uri*)rstatement->subject));
+  if(rstatement->subject->type == RAPTOR_TERM_TYPE_BLANK) {
+    node = librdf_new_node_from_blank_identifier(world, rstatement->subject->value.blank.string);
+  } else if(rstatement->subject->type == RAPTOR_TERM_TYPE_URI) {
+    node = librdf_new_node_from_uri_string(world,
+                                           librdf_uri_as_string((librdf_uri*)rstatement->subject->value.uri));
   } else {
     librdf_log(world,
                0, LIBRDF_LOG_ERROR, LIBRDF_FROM_QUERY, NULL,
                "Unknown Raptor subject identifier type %d",
-               rstatement->subject_type);
+               rstatement->subject->type);
     goto fail;
   }
 
@@ -913,25 +1012,15 @@ librdf_query_rasqal_query_results_update_statement(void* context)
   librdf_statement_set_subject(scontext->statement, node);
 
   /* predicate */
-  
-  if(rstatement->predicate_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
-    /* FIXME - but only a little
-     * Do I really need to do log10(ordinal) [or /10 and count] + 1 ? 
-     * See also librdf_heuristic_gen_name for some code to repurpose.
-     */
-    char ordinal_buffer[100]; 
-    int ordinal=*(int*)rstatement->predicate;
-    sprintf(ordinal_buffer, "http://www.w3.org/1999/02/22-rdf-syntax-ns#_%d", ordinal);
-    
-    node=librdf_new_node_from_uri_string(world, (const unsigned char*)ordinal_buffer);
-  } else if (rstatement->predicate_type == RAPTOR_IDENTIFIER_TYPE_PREDICATE ||
-             rstatement->predicate_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE) {
-    node=librdf_new_node_from_uri_string(world,
-                                         librdf_uri_as_string((librdf_uri*)rstatement->predicate));
+
+  if(rstatement->predicate->type == RAPTOR_TERM_TYPE_URI) {
+    node = librdf_new_node_from_uri_string(world,
+                                           librdf_uri_as_string((librdf_uri*)rstatement->predicate->value.uri));
   } else {
     librdf_log(world,
                0, LIBRDF_LOG_ERROR, LIBRDF_FROM_QUERY, NULL,
-               "Unknown Raptor predicate identifier type %d", rstatement->predicate_type);
+               "Unknown Raptor predicate identifier type %d",
+               rstatement->predicate->type);
     goto fail;
   }
 
@@ -945,32 +1034,22 @@ librdf_query_rasqal_query_results_update_statement(void* context)
   librdf_statement_set_predicate(scontext->statement, node);
   
   /* object */
-  
-  if(rstatement->object_type == RAPTOR_IDENTIFIER_TYPE_LITERAL ||
-     rstatement->object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL) {
-    int is_xml_literal = (rstatement->object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL);
-    librdf_uri *datatype_uri=(librdf_uri*)rstatement->object_literal_datatype;
-    
-    if(is_xml_literal)
-      node=librdf_new_node_from_literal(world,
-                                        (const unsigned char*)rstatement->object,
-                                        (const char*)rstatement->object_literal_language,
-                                        is_xml_literal);
-    else
-      node=librdf_new_node_from_typed_literal(world,
-                                              (const unsigned char*)rstatement->object,
-                                              (const char*)rstatement->object_literal_language,
-                                              datatype_uri);
-    
-  } else if(rstatement->object_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS) {
-    node=librdf_new_node_from_blank_identifier(world, (const unsigned char*)rstatement->object);
-  } else if(rstatement->object_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE) {
-    node=librdf_new_node_from_uri_string(world,
-                                         librdf_uri_as_string((librdf_uri*)rstatement->object));
+
+  if(rstatement->object->type == RAPTOR_TERM_TYPE_LITERAL) {
+    node = librdf_new_node_from_typed_literal(world,
+                                              rstatement->object->value.literal.string,
+                                              (const char*)rstatement->object->value.literal.language,
+                                              (librdf_uri*)rstatement->object->value.literal.datatype);
+  } else if(rstatement->object->type == RAPTOR_TERM_TYPE_BLANK) {
+    node = librdf_new_node_from_blank_identifier(world, rstatement->object->value.blank.string);
+  } else if(rstatement->object->type == RAPTOR_TERM_TYPE_URI) {
+    node = librdf_new_node_from_uri_string(world,
+                                           librdf_uri_as_string((librdf_uri*)rstatement->object->value.uri));
   } else {
     librdf_log(world,
                0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
-               "Unknown Raptor object identifier type %d", rstatement->object_type);
+               "Unknown Raptor object identifier type %d",
+               rstatement->object->type);
     goto fail;
   }
 
@@ -1067,11 +1146,8 @@ librdf_query_rasqal_results_as_stream(librdf_query_results* query_results)
   scontext->query=query;
   scontext->qcontext=context;
 
-  if(librdf_query_rasqal_query_results_update_statement(scontext)) {
-    librdf_query_rasqal_query_results_finished((void*)scontext);
-    return NULL;
-  }
-  
+  librdf_query_rasqal_query_results_update_statement(scontext);
+
   stream=librdf_new_stream(query->world,
                            (void*)scontext,
                            &librdf_query_rasqal_query_results_end_of_stream,
@@ -1089,50 +1165,32 @@ librdf_query_rasqal_results_as_stream(librdf_query_results* query_results)
 
 static librdf_query_results_formatter*
 librdf_query_rasqal_new_results_formatter(librdf_query_results* query_results,
-                                          const char *name, 
-                                          librdf_uri* uri)
+                                          const char *name,
+                                          const char *mime_type, 
+                                          librdf_uri* format_uri)
 {
+  rasqal_world* rasqal_world_ptr;
   rasqal_query_results_formatter* formatter;
   librdf_query_results_formatter* qrf;
 
-  formatter=rasqal_new_query_results_formatter(query_results->query->world->rasqal_world_ptr,
-                                               name, (raptor_uri*)uri);
+  rasqal_world_ptr = query_results->query->world->rasqal_world_ptr;
+
+  formatter = rasqal_new_query_results_formatter(rasqal_world_ptr,
+                                                 name,
+                                                 mime_type,
+                                                 (raptor_uri*)format_uri);
   if(!formatter)
     return NULL;
 
-  qrf=(librdf_query_results_formatter*)LIBRDF_MALLOC(query_results_formatter, 
-                                                     sizeof(librdf_query_results_formatter));
+  qrf = (librdf_query_results_formatter*)LIBRDF_MALLOC(query_results_formatter, 
+                                                       sizeof(*qrf));
   if(!qrf) {
     rasqal_free_query_results_formatter(formatter);
     return NULL;
   }
 
-  qrf->query_results=query_results;
-  qrf->formatter=formatter;
-  return qrf;
-}
-
-static librdf_query_results_formatter*
-librdf_query_rasqal_new_results_formatter_by_mime_type(librdf_query_results* query_results,
-                                                       const char *mime_type)
-{
-  rasqal_query_results_formatter* formatter;
-  librdf_query_results_formatter* qrf;
-
-  formatter=rasqal_new_query_results_formatter_by_mime_type(query_results->query->world->rasqal_world_ptr,
-                                                            mime_type);
-  if(!formatter)
-    return NULL;
-
-  qrf=(librdf_query_results_formatter*)LIBRDF_MALLOC(query_results_formatter, 
-                                                     sizeof(librdf_query_results_formatter));
-  if(!qrf) {
-    rasqal_free_query_results_formatter(formatter);
-    return NULL;
-  }
-
-  qrf->query_results=query_results;
-  qrf->formatter=formatter;
+  qrf->query_results = query_results;
+  qrf->formatter = formatter;
   return qrf;
 }
 
@@ -1191,60 +1249,61 @@ librdf_query_rasqal_register_factory(librdf_query_factory *factory)
   factory->results_as_stream                  = librdf_query_rasqal_results_as_stream;
 
   factory->new_results_formatter              = librdf_query_rasqal_new_results_formatter;
-  factory->new_results_formatter_by_mime_type = librdf_query_rasqal_new_results_formatter_by_mime_type;
   factory->free_results_formatter             = librdf_query_rasqal_free_results_formatter;
   factory->results_formatter_write            = librdf_query_rasqal_results_formatter_write;
 }
 
 
-void
+int
 librdf_query_rasqal_constructor(librdf_world *world)
 {
   unsigned int i;
 
-  world->rasqal_world_ptr=rasqal_new_world();
   if(!world->rasqal_world_ptr) {
-    LIBRDF_FATAL1(world, LIBRDF_FROM_QUERY, "failed to initialize rasqal");
-    return;
+    world->rasqal_world_ptr=rasqal_new_world();
+    world->rasqal_world_allocated_here = 1;
+
+    if(!world->rasqal_world_ptr) {
+      LIBRDF_FATAL1(world, LIBRDF_FROM_QUERY, "failed to initialize rasqal");
+      return 1;
+    }
+
+    /* Make sure rasqal works with the same raptor instance as everyone else. */
+    rasqal_world_set_raptor(world->rasqal_world_ptr, world->raptor_world_ptr);
+
+    if(rasqal_world_open(world->rasqal_world_ptr)) {
+      LIBRDF_FATAL1(world, LIBRDF_FROM_QUERY, "failed to initialize rasqal");
+      return 1;
+    }
   }
 
-  /* Make sure rasqal works with the same raptor instance as everyone else.
-   * In ifndef RAPTOR_V2_AVAILABLE case, raptor_world_ptr is NULL
-   * -> ok to use with rasqal_world_set_raptor(),
-   * rasqal should also have been built without RAPTOR_V2_AVAILABLE ->
-   * rasqal_world_open() uses raptor_init() which just increments
-   * the refcount on raptor instance already created in librdf_raptor_init().
-   */
-#ifdef RAPTOR_V2_AVAILABLE
-  rasqal_world_set_raptor(world->rasqal_world_ptr, world->raptor_world_ptr);
-  
-  if(rasqal_world_open(world->rasqal_world_ptr)) {
-    LIBRDF_FATAL1(world, LIBRDF_FROM_QUERY, "failed to initialize rasqal");
-    return;
-  }
-#endif
-  
+
   rasqal_set_triples_source_factory(world->rasqal_world_ptr, rasqal_redland_register_triples_source_factory, world);
 
-  /* enumerate from query language 1, so the default parser 0 is done last */
-  for(i=1; 1; i++) {
-    const char *language_name=NULL;
-    const unsigned char *uri_string=NULL;
+  /* enumerate from query language 1, so the default query language 0
+   * is done last 
+   */
+  for(i = 1; 1; i++) {
+    const char *language_name = NULL;
+    const unsigned char *uri_string = NULL;
 
-    if(rasqal_languages_enumerate(world->rasqal_world_ptr, i, &language_name, NULL, &uri_string)) {
-      /* reached the end of the parsers, now register the default one */
-      i=0;
-      if(rasqal_languages_enumerate(world->rasqal_world_ptr, i, &language_name, NULL, &uri_string)) {
-        /* error - should really return an error code or fail with librdf_fatal() */
-        return;
+    if(rasqal_languages_enumerate(world->rasqal_world_ptr, i, &language_name,
+                                  NULL, &uri_string)) {
+      /* reached the end of the query languages, now register the default one */
+      i = 0;
+
+      if(rasqal_languages_enumerate(world->rasqal_world_ptr, i, &language_name,
+                                    NULL, &uri_string)) {
+        LIBRDF_FATAL1(world, LIBRDF_FROM_QUERY, "failed to initialize rasqal");
+        return 1;
       }
     }
 
-  librdf_query_register_factory(world, language_name, uri_string,
-                                &librdf_query_rasqal_register_factory);
-  
-  if(!i) /* registered default query, end */
-    break;
+    librdf_query_register_factory(world, language_name, uri_string,
+                                  &librdf_query_rasqal_register_factory);
+    
+    if(!i) /* registered default query, end */
+      break;
   }
 
 
@@ -1273,13 +1332,14 @@ librdf_query_rasqal_constructor(librdf_world *world)
   }
 #endif
 
+  return 0;
 }
 
 
 void
 librdf_query_rasqal_destructor(librdf_world *world)
 {
-  if(world->rasqal_world_ptr) {
+  if(world->rasqal_world_ptr && world->rasqal_world_allocated_here) {
     rasqal_free_world(world->rasqal_world_ptr);
     world->rasqal_world_ptr=NULL;
   }
