@@ -64,11 +64,7 @@ librdf_serializer_raptor_init(librdf_serializer *serializer, void *context)
   scontext->serializer = serializer;
   scontext->serializer_name=scontext->serializer->factory->name;
 
-#ifdef RAPTOR_V2_AVAILABLE
-  scontext->rdf_serializer = raptor_new_serializer_v2(serializer->world->raptor_world_ptr, scontext->serializer_name);
-#else
-  scontext->rdf_serializer = raptor_new_serializer(scontext->serializer_name);
-#endif
+  scontext->rdf_serializer = raptor_new_serializer(serializer->world->raptor_world_ptr, scontext->serializer_name);
   if(!scontext->rdf_serializer)
     return 1;
 
@@ -107,7 +103,7 @@ librdf_serializer_raptor_get_feature(void *context, librdf_uri* feature) {
   librdf_serializer_raptor_context* scontext=(librdf_serializer_raptor_context*)context;
   unsigned char intbuffer[20]; /* FIXME */
   unsigned char *uri_string;
-  raptor_feature feature_i;
+  raptor_option feature_i;
   
   if(!feature)
     return NULL;
@@ -116,13 +112,13 @@ librdf_serializer_raptor_get_feature(void *context, librdf_uri* feature) {
   if(!uri_string)
     return NULL;
   
-#ifdef RAPTOR_V2_AVAILABLE
-  feature_i = raptor_feature_from_uri_v2(scontext->serializer->world->raptor_world_ptr, (raptor_uri*)feature);
-#else
-  feature_i = raptor_feature_from_uri((raptor_uri*)feature);
-#endif
+  feature_i = raptor_world_get_option_from_uri(scontext->serializer->world->raptor_world_ptr, (raptor_uri*)feature);
+
   if((int)feature_i >= 0) {
-    int value=raptor_serializer_get_feature(scontext->rdf_serializer, feature_i);
+    int value;
+
+    raptor_serializer_get_option(scontext->rdf_serializer, feature_i, NULL, 
+                                 &value);
     sprintf((char*)intbuffer, "%d", value);
     return librdf_new_node_from_typed_literal(scontext->serializer->world,
                                               intbuffer, NULL, NULL);
@@ -137,18 +133,15 @@ librdf_serializer_raptor_set_feature(void *context,
                                      librdf_uri *feature, librdf_node* value) 
 {
   librdf_serializer_raptor_context* scontext=(librdf_serializer_raptor_context*)context;
-  raptor_feature feature_i;
+  raptor_option feature_i;
   const unsigned char* value_s;
   
   if(!feature)
     return 1;
 
   /* try a raptor feature */
-#ifdef RAPTOR_V2_AVAILABLE
-  feature_i = raptor_feature_from_uri_v2(scontext->serializer->world->raptor_world_ptr, (raptor_uri*)feature);
-#else
-  feature_i = raptor_feature_from_uri((raptor_uri*)feature);
-#endif
+  feature_i = raptor_world_get_option_from_uri(scontext->serializer->world->raptor_world_ptr, (raptor_uri*)feature);
+
   if((int)feature_i < 0)
     return 1;
   
@@ -157,8 +150,8 @@ librdf_serializer_raptor_set_feature(void *context,
   
   value_s=(const unsigned char*)librdf_node_get_literal_value(value);
 
-  return raptor_serializer_set_feature_string(scontext->rdf_serializer,
-                                              feature_i, value_s);
+  return raptor_serializer_set_option(scontext->rdf_serializer, feature_i,
+                                      (const char *)value_s, 0);
 }
 
 
@@ -166,99 +159,89 @@ static int
 librdf_serializer_raptor_set_namespace(void* context,
                                        librdf_uri *uri, const char *prefix) 
 {
-  librdf_serializer_raptor_context* scontext=(librdf_serializer_raptor_context*)context;
-  return raptor_serialize_set_namespace(scontext->rdf_serializer, 
-                                        (raptor_uri*)uri, (const unsigned char*)prefix);
+  librdf_serializer_raptor_context* scontext = (librdf_serializer_raptor_context*)context;
+
+  return raptor_serializer_set_namespace(scontext->rdf_serializer,
+                                         (raptor_uri*)uri,
+                                         (const unsigned char*)prefix);
 }
   
 
 static int
 librdf_serializer_raptor_serialize_statement(raptor_serializer *rserializer,
-                                             librdf_statement* statement)
+                                             librdf_statement* statement,
+                                             librdf_node* graph)
 {
+  int rc = 1;
   raptor_statement rstatement;
-  librdf_node *subject=librdf_statement_get_subject(statement);
-  librdf_node *predicate=librdf_statement_get_predicate(statement);
-  librdf_node *object=librdf_statement_get_object(statement);
+  librdf_node *subject, *predicate, *object;
+
+  raptor_statement_init(&rstatement, raptor_serializer_get_world(rserializer));
+
+  subject = librdf_statement_get_subject(statement);
+  predicate = librdf_statement_get_predicate(statement);
+  object = librdf_statement_get_object(statement);
 
   if(librdf_node_is_blank(subject)) {
-    rstatement.subject=librdf_node_get_blank_identifier(subject);
-    rstatement.subject_type=RAPTOR_IDENTIFIER_TYPE_ANONYMOUS;
+    rstatement.subject = raptor_new_term_from_blank(raptor_serializer_get_world(rserializer),
+                                                    librdf_node_get_blank_identifier(subject));
   } else if(librdf_node_is_resource(subject)) {
-    rstatement.subject=(raptor_uri*)librdf_node_get_uri(subject);
-    rstatement.subject_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
+    rstatement.subject = raptor_new_term_from_uri(raptor_serializer_get_world(rserializer),
+                                                  (raptor_uri*)librdf_node_get_uri(subject));
   } else {
-    librdf_log(statement->world,
-               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER, NULL,
-               "Do not know how to serialize triple subject type %d",
-               librdf_node_get_type(subject));
-    return 1;
+    /* FIXME - what to log here if anything? */
+    goto exit;
   }
 
   if(!librdf_node_is_resource(predicate)) {
-    librdf_log(statement->world,
-               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER, NULL,
-               "Do not know how to serialize triple predicate type %d",
-               librdf_node_get_type(predicate));
-    return 1;
+    /* FIXME - what to log here if anything? */
+    goto exit;
   }
 
-  rstatement.predicate=(raptor_uri*)librdf_node_get_uri(predicate);
-  rstatement.predicate_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
+  rstatement.predicate = raptor_new_term_from_uri(raptor_serializer_get_world(rserializer),
+                                                  (raptor_uri*)librdf_node_get_uri(predicate));
 
-  rstatement.object_literal_language=NULL;
-  rstatement.object_literal_datatype=NULL;
   switch(librdf_node_get_type(object)) {
     case LIBRDF_NODE_TYPE_LITERAL:
-      rstatement.object=librdf_node_get_literal_value(object);
-      rstatement.object_type=RAPTOR_IDENTIFIER_TYPE_LITERAL;
-      
-      rstatement.object_literal_language=(const unsigned char*)librdf_node_get_literal_value_language(object);
-      rstatement.object_literal_datatype=(raptor_uri*)librdf_node_get_literal_value_datatype_uri(object);
+      rstatement.object = raptor_new_term_from_literal(raptor_serializer_get_world(rserializer),
+                                                       librdf_node_get_literal_value(object),
+                                                       (raptor_uri*)librdf_node_get_literal_value_datatype_uri(object),
+                                                       (const unsigned char*)librdf_node_get_literal_value_language(object));
       break;
 
     case LIBRDF_NODE_TYPE_BLANK:
-      rstatement.object=librdf_node_get_blank_identifier(object);
-      rstatement.object_type=RAPTOR_IDENTIFIER_TYPE_ANONYMOUS;
+      rstatement.object = raptor_new_term_from_blank(raptor_serializer_get_world(rserializer),
+                                                     librdf_node_get_blank_identifier(object));
       break;
 
     case LIBRDF_NODE_TYPE_RESOURCE:
-      rstatement.object=librdf_node_get_uri(object);
-      rstatement.object_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
+      rstatement.object = raptor_new_term_from_uri(raptor_serializer_get_world(rserializer),
+                                                   (raptor_uri*)librdf_node_get_uri(object));
       break;
 
     case LIBRDF_NODE_TYPE_UNKNOWN:
     default:
-      librdf_log(statement->world,
-                 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER, NULL,
-                 "Do not know how to serialize triple object type %d",
-                 librdf_node_get_type(object));
-      return 1;
+    /* FIXME - what to log here if anything? */
+      goto exit;
   }
 
-  return raptor_serialize_statement(rserializer, &rstatement);
-}
+  if (graph) {
+    if(librdf_node_is_resource(graph)) {
+      rstatement.graph = raptor_new_term_from_uri(raptor_serializer_get_world(rserializer),
+                                                  (raptor_uri*)librdf_node_get_uri(graph));
+    } else {
+      /* FIXME - what to log here if anything? */
+      goto exit;
+    }
+  } else {
+    rstatement.graph = NULL;
+  }
 
+  rc = raptor_serializer_serialize_statement(rserializer, &rstatement);
 
-static void
-librdf_serializer_raptor_error_handler(void *data, raptor_locator *locator,
-                                       const char *message) 
-{
-  librdf_serializer_raptor_context* scontext=(librdf_serializer_raptor_context*)data;
-  scontext->errors++;
-
-  librdf_log_simple(scontext->serializer->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_SERIALIZER, locator, message);
-}
-
-
-static void
-librdf_serializer_raptor_warning_handler(void *data, raptor_locator *locator,
-                                         const char *message) 
-{
-  librdf_serializer_raptor_context* scontext=(librdf_serializer_raptor_context*)data;
-  scontext->warnings++;
-
-  librdf_log_simple(scontext->serializer->world, 0, LIBRDF_LOG_WARN, LIBRDF_FROM_SERIALIZER, locator, message);
+  exit:
+  raptor_free_statement(&rstatement);
+  return rc;
 }
 
 
@@ -275,31 +258,29 @@ librdf_serializer_raptor_serialize_stream_to_file_handle(void *context,
     return 1;
 
   /* start the serialize */
-  rc=raptor_serialize_start_to_file_handle(scontext->rdf_serializer, 
-                                           (raptor_uri*)base_uri, handle);
+  rc = raptor_serializer_start_to_file_handle(scontext->rdf_serializer,
+                                              (raptor_uri*)base_uri, handle);
   if(rc) {
     /* free up resources on error */
-    raptor_serialize_end(scontext->rdf_serializer);
+    raptor_serializer_serialize_end(scontext->rdf_serializer);
     return 1;
   }
 
   scontext->errors=0;
   scontext->warnings=0;
-  raptor_serializer_set_error_handler(scontext->rdf_serializer, scontext, 
-                                      librdf_serializer_raptor_error_handler);
-  raptor_serializer_set_warning_handler(scontext->rdf_serializer, scontext, 
-                                        librdf_serializer_raptor_warning_handler);
 
   rc=0;
   while(!librdf_stream_end(stream)) {
-    librdf_statement *statement=librdf_stream_get_object(stream);
-    rc=librdf_serializer_raptor_serialize_statement(scontext->rdf_serializer, 
-                                                    statement);
+    librdf_statement *statement = librdf_stream_get_object(stream);
+    librdf_node *graph = librdf_stream_get_context2(stream);
+    rc = librdf_serializer_raptor_serialize_statement(scontext->rdf_serializer,
+                                                      statement, graph);
     if(rc)
       break;
     librdf_stream_next(stream);
   }
-  raptor_serialize_end(scontext->rdf_serializer);
+
+  raptor_serializer_serialize_end(scontext->rdf_serializer);
 
   return rc;
 }
@@ -343,15 +324,16 @@ librdf_serializer_raptor_serialize_stream_to_counted_string(void *context,
     return NULL;
 
   /* start the serialize */
-  iostr=raptor_new_iostream_to_string(&string, &string_length,
-                                      malloc);
+  iostr = raptor_new_iostream_to_string(raptor_serializer_get_world(scontext->rdf_serializer),
+                                        &string, &string_length,
+                                        malloc);
   if(!iostr) {
     free(string);
     return NULL;
   }
-    
-  rc=raptor_serialize_start(scontext->rdf_serializer, 
-                            (raptor_uri*)base_uri, iostr);
+
+  rc = raptor_serializer_start_to_iostream(scontext->rdf_serializer,
+                                           (raptor_uri*)base_uri, iostr);
 
   if(rc) {
     raptor_free_iostream(iostr);
@@ -361,21 +343,23 @@ librdf_serializer_raptor_serialize_stream_to_counted_string(void *context,
     
   scontext->errors=0;
   scontext->warnings=0;
-  raptor_serializer_set_error_handler(scontext->rdf_serializer, scontext, 
-                                      librdf_serializer_raptor_error_handler);
-  raptor_serializer_set_warning_handler(scontext->rdf_serializer, scontext, 
-                                        librdf_serializer_raptor_warning_handler);
 
   rc=0;    
   while(!librdf_stream_end(stream)) {
-    librdf_statement *statement=librdf_stream_get_object(stream);
-    rc=librdf_serializer_raptor_serialize_statement(scontext->rdf_serializer, 
-                                                    statement);
+    librdf_statement *statement = librdf_stream_get_object(stream);
+    librdf_node *graph = librdf_stream_get_context2(stream);
+    rc = librdf_serializer_raptor_serialize_statement(scontext->rdf_serializer,
+                                                      statement, graph);
     if(rc)
       break;
     librdf_stream_next(stream);
   }
-  raptor_serialize_end(scontext->rdf_serializer);
+  raptor_serializer_serialize_end(scontext->rdf_serializer);
+
+  /* raptor2 raptor_serialize_start_to_iostream() does not take
+   * ownership of iostream 
+   */
+  raptor_free_iostream(iostr);
 
   if(rc) {
     free(string);
@@ -428,8 +412,8 @@ librdf_serializer_raptor_serialize_stream_to_iostream(void *context,
     return 1;
 
   /* start the serialize */
-  rc=raptor_serialize_start(scontext->rdf_serializer, 
-                            (raptor_uri*)base_uri, iostr);
+  rc = raptor_serializer_start_to_iostream(scontext->rdf_serializer,
+                                           (raptor_uri*)base_uri, iostr);
 
   if(rc) {
     raptor_free_iostream(iostr);
@@ -438,21 +422,23 @@ librdf_serializer_raptor_serialize_stream_to_iostream(void *context,
     
   scontext->errors=0;
   scontext->warnings=0;
-  raptor_serializer_set_error_handler(scontext->rdf_serializer, scontext, 
-                                      librdf_serializer_raptor_error_handler);
-  raptor_serializer_set_warning_handler(scontext->rdf_serializer, scontext, 
-                                        librdf_serializer_raptor_warning_handler);
 
   rc=0;
   while(!librdf_stream_end(stream)) {
-    librdf_statement *statement=librdf_stream_get_object(stream);
-    rc=librdf_serializer_raptor_serialize_statement(scontext->rdf_serializer, 
-                                                    statement);
+    librdf_statement *statement = librdf_stream_get_object(stream);
+    librdf_node *graph = librdf_stream_get_context2(stream);
+    rc = librdf_serializer_raptor_serialize_statement(scontext->rdf_serializer,
+                                                      statement, graph);
     if(rc)
       break;
     librdf_stream_next(stream);
   }
-  raptor_serialize_end(scontext->rdf_serializer);
+  raptor_serializer_serialize_end(scontext->rdf_serializer);
+
+  /* raptor1 raptor_serialize_start_to_iostream() does not take
+   * ownership of iostr
+   */
+  raptor_free_iostream(iostr);
 
   return rc;
 }
@@ -523,31 +509,27 @@ librdf_serializer_raptor_constructor(librdf_world *world)
   int i;
   
   /* enumerate from serializer 1, so the default serializer 0 is done last */
-  for(i=1; 1; i++) {
-    const char *syntax_name=NULL;
-    const char *syntax_label=NULL;
-    const char *mime_type=NULL;
-    const unsigned char *uri_string=NULL;
+  for(i = 1; 1; i++) {
+    const char *syntax_name = NULL;
+    const char *syntax_label = NULL;
+    const char *mime_type = NULL;
+    const unsigned char *uri_string = NULL;
 
-#ifdef RAPTOR_V2_AVAILABLE
-    if(raptor_serializers_enumerate_v2(world->raptor_world_ptr,
-                                       i, &syntax_name, &syntax_label, 
-                                       &mime_type, &uri_string)) {
+    const raptor_syntax_description *desc;
+
+    desc = raptor_world_get_serializer_description(world->raptor_world_ptr, i);
+
+    if(!desc) {
       /* reached the end of the serializers, now register the default one */
-      i=0;
-      raptor_serializers_enumerate_v2(world->raptor_world_ptr,
-                                      i, &syntax_name, &syntax_label,
-                                      &mime_type, &uri_string);
+      i = 0;
+      desc = raptor_world_get_serializer_description(world->raptor_world_ptr, i);
     }
-#else
-    if(raptor_serializers_enumerate(i, &syntax_name, &syntax_label, 
-                                    &mime_type, &uri_string)) {
-      /* reached the end of the serializers, now register the default one */
-      i=0;
-      raptor_serializers_enumerate(i, &syntax_name, &syntax_label,
-                                   &mime_type, &uri_string);
-    }
-#endif
+    syntax_name = desc->names[0];
+    syntax_label = desc->label;
+    if(desc->mime_types)
+      mime_type = desc->mime_types[0].mime_type;
+    if(desc->uri_strings)
+      uri_string = (const unsigned char *)desc->uri_strings[0];
     
     librdf_serializer_register_factory(world, syntax_name, syntax_label,
                                        mime_type, uri_string,
