@@ -36,7 +36,7 @@
 #include <ctype.h>
 #include <sys/types.h>
 #ifdef HAVE_STDLIB_H
-#include <stdlib.h> /* for abort() as used in errors */
+#include <stdlib.h>
 #endif
 
 #ifdef MODULAR_LIBRDF
@@ -104,7 +104,7 @@ librdf_init_storage(librdf_world *world)
 
   if (!world->storage_modules)
     world->storage_modules = raptor_new_sequence(
-        (raptor_sequence_free_handler*)lt_dlclose, NULL);
+        (raptor_data_free_handler)lt_dlclose, NULL);
 
   librdf_storage_load_all_modules(world);
 
@@ -113,6 +113,9 @@ librdf_init_storage(librdf_world *world)
   #ifdef STORAGE_MYSQL
     librdf_init_storage_mysql(world);
   #endif
+  #ifdef STORAGE_VIRTUOSO
+    librdf_init_storage_virtuoso(world);
+  #endif
   #ifdef STORAGE_POSTGRESQL
     librdf_init_storage_postgresql(world);
   #endif
@@ -169,51 +172,64 @@ librdf_free_storage_factory(librdf_storage_factory* factory)
 
 #ifdef MODULAR_LIBRDF
 
-/* pulled from raptor_memstr.c */
-static const char *  
-librdf_memstr(const char *haystack, size_t haystack_len, const char *needle)
-{
-  char c;
-  size_t needle_len;
-  const char *p;
-  
-  if(!haystack || !needle)
-    return NULL;
-  
-  if(!*needle)
-    return haystack;
-  
-  needle_len=strlen(needle);
-
-  /* loop invariant: haystack_len is always length of remaining buffer at *p */
-  for(p=haystack;
-      (haystack_len >= needle_len) && (c=*p);
-      p++, haystack_len--) {
-
-    /* check match */
-    if(!memcmp(p, needle, needle_len))
-      return p;
-  }
-  
-  return NULL;
-}
-
-
 static int
 ltdl_module_callback(const char* filename, void* data)
 {
   librdf_world* world = (librdf_world*)data;
+  const char* name = librdf_basename(filename);
+  size_t name_len = strlen(name);
+  lt_dlhandle module;
 
-  /* FIXME: Currently requiring that modules to be loaded contain "librdf_storage".
-     Not all files in .libs that contain librdf_storage are storage modules.
-     (Relevant when running "make check" locally before installing the modules.)
-     Also tries to load some storage modules multiple times. */
-  if (librdf_memstr(filename, strlen(filename), "librdf_storage")) {
-    lt_dlhandle module = librdf_storage_load_module(world, filename,
-                                                    "librdf_storage_module_register_factory");
-    if (module)
-      raptor_sequence_push(world->storage_modules, module);
+  /* Currently require that storage module files to be loaded start
+   * with the string "librdf_storage_".
+   */
+
+  if(name_len < 15 || strncmp(name, "librdf_storage_", 15)) {
+#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 2
+    LIBRDF_DEBUG3("not storage module file %s (%s)\n", name, filename);
+#endif
+    return 0;
   }
+
+  /*
+   * When compiling and testing against uninstalled modules, not all
+   * files in .libs that contain librdf_storage are storage modules.
+   * (Relevant when running "make check" locally before installing
+   * the modules.)
+   *
+   * This check when debugging LIBRDF_DEBUG attempts to load .so
+   * files when running in the source tree.  This is likely very
+   * libtool and unix specific.
+   *
+   * Rules:
+   * - Must end in '.so' or have no suffix (no . in last 3 chars)
+   * - Must not include "-" (libtool intermediate file such as
+   *   librdf_storage_mysql_la-rdf_storage_mysql)
+   */
+#ifdef LIBRDF_DEBUG
+  if(!(
+       strncmp(&name[name_len-3], ".so", 3) ||
+       (name[name_len-3]!='.' && name[name_len-2]!='.' && name[name_len-1]!='.')
+       )
+     || strchr(name, '-')
+     ) {
+#if LIBRDF_DEBUG > 2
+    LIBRDF_DEBUG3("not storage module file %s (%s)\n", name, filename);
+#endif
+    return 0;
+  }
+
+#if LIBRDF_DEBUG > 1
+  LIBRDF_DEBUG3("LOADING storage module file %s (%s)\n", name, filename);
+#endif
+
+#endif
+
+  module = librdf_storage_load_module(world, filename,
+                                      "librdf_storage_module_register_factory");
+  if(module)
+    raptor_sequence_push(world->storage_modules, module);
+
   return 0;
 }
 
@@ -263,6 +279,7 @@ librdf_storage_load_module(librdf_world *world,
   init_func_t* init;
   
   lt_dlhandle module = lt_dlopenext(lib_name);
+
   if (module) {
     init = (init_func_t*)lt_dlsym(module, init_func_name);
     if (init) {
@@ -322,7 +339,8 @@ librdf_storage_register_factory(librdf_world* world,
 #endif
 
   if(!world->storages) {
-    world->storages=raptor_new_sequence((raptor_sequence_free_handler *)librdf_free_storage_factory, NULL);
+    world->storages = raptor_new_sequence((raptor_data_free_handler)librdf_free_storage_factory, NULL);
+
     if(!world->storages)
       goto failed;
   }
@@ -356,10 +374,6 @@ librdf_storage_register_factory(librdf_world* world,
   /* Call the storage registration function on the new object */
   (*factory)(storage);
 
-#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
-  LIBRDF_DEBUG3("%s has context size %d\n", name, storage->context_length);
-#endif
-
   if(storage->version < LIBRDF_STORAGE_MIN_INTERFACE_VERSION ||
      storage->version > LIBRDF_STORAGE_MAX_INTERFACE_VERSION) {
     librdf_log(world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
@@ -367,7 +381,7 @@ librdf_storage_register_factory(librdf_world* world,
                name, storage->version,
                LIBRDF_STORAGE_MIN_INTERFACE_VERSION,
                LIBRDF_STORAGE_MAX_INTERFACE_VERSION);
-    goto failed;
+    goto tidy;
   }
   
   if(raptor_sequence_push(world->storages, storage))
@@ -481,9 +495,13 @@ librdf_new_storage(librdf_world *world,
   
   librdf_world_open(world);
 
-  factory=librdf_get_storage_factory(world, storage_name);
-  if(!factory)
+  factory = librdf_get_storage_factory(world, storage_name);
+  if(!factory) {
+    librdf_log(world, 
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "storage '%s' not found", name);
     return NULL;
+  }
 
   options_hash=librdf_new_hash(world, NULL);
   if(!options_hash)
@@ -527,9 +545,13 @@ librdf_new_storage_with_options(librdf_world *world,
   
   librdf_world_open(world);
 
-  factory=librdf_get_storage_factory(world, storage_name);
-  if(!factory)
+  factory = librdf_get_storage_factory(world, storage_name);
+  if(!factory) {
+    librdf_log(world, 
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "storage '%s' not found", name);
     return NULL;
+  }
 
   options_hash=librdf_new_hash_from_hash(options);
   if(!options_hash)
@@ -545,9 +567,11 @@ librdf_new_storage_with_options(librdf_world *world,
 
 
 /**
- * librdf_new_storage_from_storage - Copy constructor - create a new librdf_storage object from an existing one
+ * librdf_new_storage_from_storage:
  * @old_storage: the existing storage #librdf_storage to use
  *
+ * Copy constructor - create a new librdf_storage object from an existing one
+ *
  * Should create a new storage in the same context as the existing one
  * as appropriate for the storage.  For example, in a RDBMS storage
  * it would be a new database, or in on disk it would be a new
@@ -668,8 +692,9 @@ librdf_new_storage_from_factory(librdf_world *world,
 void
 librdf_free_storage(librdf_storage* storage) 
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(storage, librdf_storage);
-
+  if(!storage)
+    return;
+  
   if(--storage->usage)
     return;
 
@@ -685,6 +710,7 @@ librdf_free_storage(librdf_storage* storage)
  * @storage: #librdf_storage object
  *
  * Increment storage reference count by one.
+ *
  * This function is intended to be internal to librdf storage modules.
  **/
 void
@@ -695,11 +721,11 @@ librdf_storage_add_reference(librdf_storage *storage)
 
 
 /**
- * librdf_storage_remove_reference(libdf_storage *storage)
+ * librdf_storage_remove_reference:
  * @storage: #librdf_storage object
  *
- * Decrement storage reference count by one.
- * Free the storage if reference count becomes zero.
+ * Decrement storage reference count by one and free the storage if reference count becomes zero.
+ *
  * This function is intended to be internal to librdf storage modules.
  **/
 void
@@ -1103,7 +1129,7 @@ librdf_storage_stream_to_node_iterator_get_method(void* iterator, int flags)
 
           case LIBRDF_STATEMENT_ALL:
             default: /* error */
-              librdf_log(statement->world,
+              librdf_log(context->storage->world,
                          0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                          "Unknown statement part %d", context->want);
               node=NULL;
@@ -1115,14 +1141,14 @@ librdf_storage_stream_to_node_iterator_get_method(void* iterator, int flags)
       
     case LIBRDF_ITERATOR_GET_METHOD_GET_CONTEXT:
       if(!context->context_node) {
-        node=(librdf_node*)librdf_stream_get_context(context->stream);
+        node = librdf_stream_get_context2(context->stream);
         context->context_node=node ? librdf_new_node_from_node(node) : NULL;
       }
       node=context->context_node;
       break;
       
     default:
-      librdf_log(statement->world,
+      librdf_log(context->storage->world,
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                  "Unknown iterator method flag %d", flags);
       node=NULL;
@@ -1158,12 +1184,14 @@ librdf_storage_stream_to_node_iterator_finished(void* iterator)
 
 
 /*
- * librdf_storage_node_stream_to_node_create - Create a stream for get sources, targets or arcs methods using find_statements method
+ * librdf_storage_node_stream_to_node_create:
  * @storage: the storage object to use
  * @node1: the first node to encode in the key (or NULL if not needed)
  * @node2: the second node to encode in the key (or NULL if not needed)
  * @want: the field required from the statement
  *
+ * INTERNAL - Create a stream for get sources, targets or arcs methods using find_statements method
+ *
  * node1 and node2 cannot both be NULL
  * 
  * Return value: a new #librdf_iterator or NULL on failure
@@ -1547,7 +1575,7 @@ librdf_storage_context_remove_statement(librdf_storage* storage,
                                         librdf_statement* statement) 
 {
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, 1);
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(context, librdf_statement, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, 1);
 
   if(!storage->factory->context_remove_statement)
     return 1;
@@ -1644,8 +1672,6 @@ librdf_storage_context_serialise(librdf_storage* storage,
  * 
  * Check if a storage system supports a query language.
  * 
- * Not implemented.
- *
  * Return value: non-0 if the query is supported.
  **/
 int
@@ -1654,7 +1680,10 @@ librdf_storage_supports_query(librdf_storage* storage, librdf_query *query)
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, 0);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, librdf_query, 0);
 
-  return 0;
+  if(storage->factory->supports_query)
+    return storage->factory->supports_query(storage, query);
+  else
+    return 0;
 }
 
 
@@ -1665,8 +1694,6 @@ librdf_storage_supports_query(librdf_storage* storage, librdf_query *query)
  * 
  * Run the given query against the storage.
  * 
- * Not implemented.
- *
  * Return value: #librdf_query_results or NULL on failure
  **/
 librdf_query_results*
@@ -1675,7 +1702,10 @@ librdf_storage_query_execute(librdf_storage* storage, librdf_query *query)
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, NULL);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, librdf_query, NULL);
 
-  return NULL;
+  if(storage->factory->supports_query)
+    return storage->factory->query_execute(storage, query);
+  else
+    return NULL;
 }
 
 
@@ -1958,7 +1988,11 @@ main(int argc, char *argv[])
   /* triples of arguments to librdf_new_storage */
   const char* const storages[] = {
 	"memory", NULL, "contexts='yes'",
+#ifdef HAVE_BDB_HASH
 	"hashes", "test", "hash-type='bdb',dir='.',write='yes',new='yes',contexts='yes'",
+#else
+	"hashes", "test", "hash-type='memory',write='yes',new='yes',contexts='yes'",
+#endif
     #ifdef STORAGE_TREES
 	    "trees", "test", "contexts='yes'",
     #endif
