@@ -2,7 +2,7 @@
  *
  * rdfproc.c - Redland RDF command processor
  *
- * Copyright (C) 2000-2007, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2010, David Beckett http://www.dajobe.org/
  * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -129,14 +129,14 @@ static command commands[]={
 
 #ifdef HAVE_GETOPT_LONG
 #define HELP_TEXT(short, long, description) "  -" #short ", --" long "  " description
-#define HELP_ARG(short, long) "--" #long
+#define HELP_ARG(short, long) "-" #short " / --" #long
 #else
 #define HELP_TEXT(short, long, description) "  -" #short "  " description
 #define HELP_ARG(short, long) "-" #short
 #endif
 
 
-#define GETOPT_STRING "chno:pr:s:t:Tv"
+#define GETOPT_STRING "chno:pqr:s:t:TvV"
 
 #ifdef HAVE_GETOPT_LONG
 static struct option long_options[] =
@@ -147,11 +147,13 @@ static struct option long_options[] =
   {"new", 0, 0, 'n'},
   {"output", 1, 0, 'o'},
   {"password", 0, 0, 'p'},
+  {"quiet", 0, 0, 'q'},
   {"results", 1, 0, 'r'},
   {"storage", 1, 0, 's'},
   {"storage-options", 1, 0, 't'},
   {"transactions", 0, 0, 'T'},
   {"version", 0, 0, 'v'},
+  {"verbose", 0, 0, 'V'},
   {NULL, 0, 0, 0}
 };
 #endif
@@ -165,10 +167,6 @@ static const char *default_storage_options="hash-type='bdb',dir='.'";
 static int
 log_handler(void *user_data, librdf_log_message *message) 
 {
-#ifdef RAPTOR_V2_AVAILABLE
-  librdf_world* world = (librdf_world*)user_data;
-#endif
-
   /* int code=message->code; */ /* The error code */
   raptor_locator *locator=(raptor_locator*)(message->locator);
 
@@ -182,11 +180,7 @@ log_handler(void *user_data, librdf_log_message *message)
     fprintf(stderr, "%s: Error - ", program);
 
   if(locator) { /* && message->facility == LIBRDF_FROM_PARSER) */
-#ifdef RAPTOR_V2_AVAILABLE
-    raptor_print_locator_v2(librdf_world_get_raptor(world), stderr, locator);
-#else
-    raptor_print_locator(stderr, locator); 
-#endif
+    raptor_locator_print(locator, stderr);
     fputc(':', stderr);
     fputc(' ', stderr);
  }
@@ -268,6 +262,7 @@ main(int argc, char *argv[])
   int usage=0;
   int help=0;
   int is_new=0;
+  int verbosity = 1;
   char *identifier=NULL;
   char *cmd=NULL;
   char *name;
@@ -352,14 +347,25 @@ main(int argc, char *argv[])
 
       case 'o':
         if(optarg) {
-          if(!strcmp(optarg, "simple"))
+          if(!strcmp(optarg, "help")) {
+            fprintf(stderr, "%s: Valid serializers are:\n", program);
+            for(i=0; 1; i++) {
+              const raptor_syntax_description* desc;
+              
+              desc = librdf_serializer_get_description(world, i);
+              if(!desc)
+                break;
+              
+              printf("  %-20s  %s\n", desc->names[0], desc->label);
+            }
+            exit(0);
+          } else if(!strcmp(optarg, "simple")) {
+            fprintf(stderr, "%s: WARNING: The deprecated 'simple' output type has been replaced with N-Triples\n", program);
             output_serializer=NULL;
-          else {
+          } else {
             output_serializer=librdf_new_serializer(world, optarg, NULL, NULL);
             if(!output_serializer) {
-              fprintf(stderr, "%s: unknown output serializer `%s' for `" HELP_ARG(o, output) "'\n",
-                      program, optarg);
-              fprintf(stderr, "Valid arguments are:\n  `simple'   for a simple format (default)\n  `ntriples' for N-Triples\n");
+              fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(o, output) "'\nTry '%s " HELP_ARG(o, output) " help' for a list of valid serializers\n", program, optarg, program);
               usage=1;
             } else {
               output_storage = librdf_new_storage(world, NULL, NULL, NULL);
@@ -422,27 +428,29 @@ main(int argc, char *argv[])
         }
         break;
 
+     case 'q':
+      verbosity=0;
+      break;
+
       case 'r':
         if(optarg) {
           if(!strcmp(optarg, "help")) {
             fprintf(stderr, "%s: Valid query result formats are:\n", program);
             for(i=0; 1; i++) {
-              const char *format_name;
-              const char *format_label;
-              if(rasqal_query_results_formats_enumerate(world->rasqal_world_ptr,
-                                                        i,
-                                                        &format_name, 
-                                                        &format_label,
-                                                        NULL, NULL, NULL))
+              const raptor_syntax_description* desc;
+              
+              desc = librdf_query_results_formats_get_description(world, i);
+              if(!desc)
                 break;
-              printf("  %-20s  %s\n", format_name, format_label);
+              
+              printf("  %-20s  %s\n", desc->names[0], desc->label);
             }
             exit(0);
           } else {
-            if(rasqal_query_results_formats_check(world->rasqal_world_ptr, optarg, NULL, NULL))
+            if(librdf_query_results_formats_check(world, optarg, NULL, NULL))
               results_format=optarg;
             else {
-              fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(r, results) "'\nTry '%s " HELP_ARG(r, results) " help' for a list of valid formats\n",
+              fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(r, results) "'\nTry '%s " HELP_ARG(r, results) " help' for a list of valid query result formats\n",
                       program, optarg, program);
               usage=1;
             }
@@ -452,8 +460,22 @@ main(int argc, char *argv[])
         
       case 's':
         if(optarg) {
+          if(!strcmp(optarg, "help")) {
+            fprintf(stderr, "%s: Valid storage names are:\n", program);
+            for(i=0; 1; i++) {
+              const char *format_name;
+              const char *format_label;
+              if(librdf_storage_enumerate(world, i,
+                                          &format_name, 
+                                          &format_label))
+                break;
+              printf("  %-10s  %s\n", format_name, format_label);
+            }
+            exit(0);
+          }
           if(!librdf_get_storage_factory(world, optarg)) {
-            fprintf(stderr, "%s: invalid storage `%s'\n", program, optarg);
+            fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(s, storage) "'\nTry '%s " HELP_ARG(s, storage) " help' for a list of valid storages\n",
+                    program, optarg, program);
             usage=1;
           } else
             storage_name=optarg;
@@ -472,6 +494,10 @@ main(int argc, char *argv[])
         fputs(librdf_version_string, stdout);
         fputc('\n', stdout);
         exit(0);
+
+     case 'V':
+      verbosity++;
+      break;
     }
     
   }
@@ -541,27 +567,36 @@ main(int argc, char *argv[])
     printf(title_format_string, librdf_version_string);
     puts(librdf_short_copyright_string);
     puts("Utility for processing RDF using the Redland library.");
-    puts("\nMain options:");
+    puts("\nOptions:");
     puts(HELP_TEXT(c, "contexts        ", "Use Redland contexts"));
     puts(HELP_TEXT(h, "help            ", "Print this help, then exit"));
     puts(HELP_TEXT(n, "new             ", "Create a new store (default no)"));
-    puts(HELP_TEXT(o, "output FORMAT   ", "Set the triple output format to one of:"));
-    puts("    simple                  A simple format (default)\n    ntriples                N-Triples\n    rdfxml                  RDF/XML");
-    puts(HELP_TEXT(p, "password        ", "Read storage option 'password' from standard input"));
-    puts(HELP_TEXT(r, "results FORMAT  ", "Set the query results format"));
+    puts(HELP_TEXT(o, "output FORMAT   ", "Set the triple output format"));
     for(i=0; 1; i++) {
-      const char *help_name;
-      const char *help_label;
-      if(librdf_query_results_formats_enumerate(world, i, 
-                                                &help_name, &help_label, 
-                                                NULL, NULL))
+      const raptor_syntax_description* desc;
+      
+      desc = librdf_serializer_get_description(world, i);
+      if(!desc)
         break;
-      printf("    %-10s              %s", help_name, help_label);
-      if(!i)
+      
+      printf("    %-15s         %s", desc->names[0], desc->label);
+      if(!strcmp(desc->names[0], query_graph_serializer_syntax_name))
         puts(" (default)");
       else
         putchar('\n');
     }
+    puts(HELP_TEXT(p, "password        ", "Read storage option 'password' from standard input"));
+    puts(HELP_TEXT(q, "quiet           ", "Do not print information messages"));
+    puts(HELP_TEXT(r, "results FORMAT  ", "Set the query results format (no default)"));
+    for(i=0; 1; i++) {
+      const raptor_syntax_description* desc;
+      
+      desc = librdf_query_results_formats_get_description(world, i);
+      if(!desc)
+        break;
+
+      printf("    %-10s              %s\n", desc->names[0], desc->label);
+    }
     puts(HELP_TEXT(s, "storage TYPE    ", "Set the graph storage type"));
     for(i=0; 1; i++) {
       const char *help_name;
@@ -569,13 +604,14 @@ main(int argc, char *argv[])
       if(librdf_storage_enumerate(world, i, &help_name, &help_label))
         break;
       printf("    %-10s              %s", help_name, help_label);
-      if(!i)
+      if(!strcmp(help_name, default_storage_name))
         puts(" (default)");
       else
         putchar('\n');
     }
     printf(HELP_TEXT(t, "storage-options OPTIONS\n                        ", "Storage options (default \"%s\")\n"), default_storage_options);
     puts(HELP_TEXT(v, "version         ", "Print the Redland version"));
+    puts(HELP_TEXT(V, "verbose         ", "Increase message verbosity"));
     puts("\nCommands:");
     puts("  parse FILE|URI [SYNTAX [BASEURI [CONTEXT]]]");
     puts("  parse-stream FILE|URI [SYNTAX [BASEURI [CONTEXT]]]");
@@ -620,6 +656,14 @@ main(int argc, char *argv[])
     librdf_hash_put_strings(options, "write", "yes");
     if(is_new)
       librdf_hash_put_strings(options, "new", "yes");
+  } else {
+    if(is_new) {
+      fprintf(stderr,
+              "%s: Cannot empty a new store during read-only command '%s'\n"
+              "%s: Retry without the " HELP_ARG(n, new) " option\n",
+              program,  cmd, program);
+      return(1);
+    }
   }
 
   librdf_hash_from_string(options, storage_options);
@@ -656,9 +700,16 @@ main(int argc, char *argv[])
   query=NULL;
   results=NULL;
 
+  rc=0;
+
   switch(type) {
     case CMD_PRINT:
-      librdf_model_print(model, stdout);
+      {
+        raptor_iostream *iostr;
+        iostr = raptor_new_iostream_to_file_handle(world->raptor_world_ptr, stdout);
+        librdf_model_write(model, iostr);
+        raptor_free_iostream(iostr);
+      }
       break;
 
 
@@ -682,9 +733,10 @@ main(int argc, char *argv[])
         librdf_free_uri(uri);
         break;
       }
-      fprintf(stdout, "%s: Parsing URI %s with %s parser\n", program,
-              librdf_uri_as_string(uri), 
-              (argc > 1) ? argv[1] : "default");
+      if(verbosity)
+        fprintf(stderr, "%s: Parsing URI %s with %s parser\n", program,
+                librdf_uri_as_string(uri), 
+                (argc > 1) ? argv[1] : "default");
       
       if(argc >= 3 && argv[2]) {
         base_uri=NULL;
@@ -696,8 +748,9 @@ main(int argc, char *argv[])
             break;
           }
         }
-        fprintf(stderr, "%s: Using base URI %s\n", program,
-                (base_uri ? (const char*)librdf_uri_as_string(base_uri) : "NULL"));
+        if(verbosity)
+          fprintf(stderr, "%s: Using base URI %s\n", program,
+                  (base_uri ? (const char*)librdf_uri_as_string(base_uri) : "NULL"));
 
         target=NULL; /* context node */
         if(argc >= 4 && argv[3]) {
@@ -706,8 +759,9 @@ main(int argc, char *argv[])
           else
             target=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[3]);
         }
-        fprintf(stderr, "%s: Using context node %s\n", program,
-                (argv[3] ? argv[3] : "NULL"));
+        if(verbosity)
+          fprintf(stderr, "%s: Using context node %s\n", program,
+                  (argv[3] ? argv[3] : "NULL"));
       } else
         base_uri=librdf_new_uri_from_uri(uri);
 
@@ -747,7 +801,8 @@ main(int argc, char *argv[])
           target=NULL;
         }
         
-        fprintf(stderr, "%s: Added %d triples\n", program, count);
+        if(verbosity)
+          fprintf(stderr, "%s: Added %d triples\n", program, count);
         rc=1;
       }
 
@@ -769,7 +824,7 @@ main(int argc, char *argv[])
 
         warning_count_node=librdf_parser_get_feature(parser, warning_count_uri);
         if(warning_count_node) {
-          warning_count=atoi((const char*)librdf_node_get_literal_value(error_count_node));
+          warning_count=atoi((const char*)librdf_node_get_literal_value(warning_count_node));
           librdf_free_node(warning_count_node);
         } else {
           fprintf(stderr, "%s: Could not get parsing warning count\n", program);
@@ -777,13 +832,16 @@ main(int argc, char *argv[])
         }
 
         if((error_count >=0) && (warning_count >=0) &&
-           error_count+warning_count >0)
-          fprintf(stderr,
+           error_count+warning_count >0) {
+          fprintf(stdout,
                   "%s: The parsing returned %d errors and %d warnings\n", 
                   program, error_count, warning_count);
+        }
+        
          
         librdf_free_uri(error_count_uri);
         librdf_free_uri(warning_count_uri);
+        rc = (error_count == 0) ? 0 : 1;
       }
       
       librdf_free_parser(parser);
@@ -815,7 +873,8 @@ main(int argc, char *argv[])
         if(strcmp(argv[1], "-")) {
           uri=librdf_new_uri(world, (const unsigned char *)argv[1]);
           if(!uri) {
-            fprintf(stderr, "%s: Failed to create URI from %s\n", program, argv[1]);
+            fprintf(stderr, "%s: Failed to create URI from %s\n", program,
+                    argv[1]);
             break;
           }
         }
@@ -829,7 +888,8 @@ main(int argc, char *argv[])
           if(strcmp(argv[3], "-")) {
             base_uri=librdf_new_uri(world, (const unsigned char *)argv[3]);
             if(!base_uri) {
-              fprintf(stderr, "%s: Failed to create base URI from %s\n", program, argv[3]);
+              fprintf(stderr, "%s: Failed to create base URI from %s\n",
+                      program, argv[3]);
               break;
             }
           }
@@ -897,7 +957,8 @@ main(int argc, char *argv[])
         }
       }
 
-      query=librdf_new_query(world, name, uri, (const unsigned char *)argv[2], NULL);
+      query=librdf_new_query(world, name, uri, (const unsigned char *)argv[2],
+                             NULL);
       if(!query) {
         fprintf(stderr, "%s: Failed to create new query %s\n", program,
                 argv[2]);
@@ -920,12 +981,16 @@ main(int argc, char *argv[])
         raptor_iostream *iostr;
         librdf_query_results_formatter *formatter;
 
-        fprintf(stdout, "%s: Formatting query result as '%s':\n", program,
-                results_format);
+        if(verbosity)
+          fprintf(stderr, "%s: Formatting query result as '%s':\n", program,
+                  results_format);
 
-        iostr=raptor_new_iostream_to_file_handle(stdout);
-        formatter=librdf_new_query_results_formatter(results,
-                                                     results_format, NULL);
+        iostr = raptor_new_iostream_to_file_handle(world->raptor_world_ptr,
+                                                   stdout);
+        formatter = librdf_new_query_results_formatter2(results,
+                                                        results_format,
+                                                        NULL /* mime type */,
+                                                        NULL /* format_uri */);
 
         librdf_query_results_formatter_write(iostr, formatter, results, 
                                              base_uri);
@@ -933,10 +998,13 @@ main(int argc, char *argv[])
         librdf_free_query_results_formatter(formatter);
         raptor_free_iostream(iostr);
       } else if(librdf_query_results_is_bindings(results)) {
-        fprintf(stdout, "%s: Query returned bindings results:\n", program);
+        if(verbosity)
+          fprintf(stderr, "%s: Query returned bindings results:\n", program);
         
         while(!librdf_query_results_finished(results)) {
-          fputs("result: [", stdout);
+          if(verbosity)
+            fputs("result: ", stdout);
+          fputs("[", stdout);
           for(i=0; i<librdf_query_results_get_bindings_count(results); i++) {
             librdf_node *value=librdf_query_results_get_binding_value(results, i);
             name=(char*)librdf_query_results_get_binding_name(results, i);
@@ -955,12 +1023,15 @@ main(int argc, char *argv[])
           librdf_query_results_next(results);
         }
 
-        fprintf(stdout, "%s: Query returned %d results\n", program, 
-                librdf_query_results_get_count(results));
+        if(verbosity)
+          fprintf(stderr, "%s: Query returned %d results\n", program, 
+                  librdf_query_results_get_count(results));
      } else if(librdf_query_results_is_boolean(results)) {
-       fprintf(stdout, "%s: Query returned boolean result: %s\n",
-               program,
-               librdf_query_results_get_boolean(results) ? "true" : "false");
+        if(verbosity)
+          fprintf(stderr, "%s: Query returned boolean result: %s\n",
+                  program,
+                  librdf_query_results_get_boolean(results) ? "true" : "false");
+        rc = !librdf_query_results_get_boolean(results);
      } else if(librdf_query_results_is_graph(results)) {
        librdf_storage* tmp_storage;
        librdf_model* tmp_model;
@@ -968,7 +1039,8 @@ main(int argc, char *argv[])
        tmp_storage=librdf_new_storage(world, NULL, NULL, NULL);
        tmp_model=librdf_new_model(world, tmp_storage, NULL);
 
-       fprintf(stdout, "%s: Query returned graph result:\n", program);
+       if(verbosity)
+         fprintf(stderr, "%s: Query returned graph result:\n", program);
        
        stream=librdf_query_results_as_stream(results);
        if(!stream) {
@@ -978,7 +1050,9 @@ main(int argc, char *argv[])
        librdf_model_add_statements(tmp_model, stream);
        librdf_free_stream(stream);
 
-       fprintf(stdout, "%s: Total %d triples\n", program, librdf_model_size(model));
+       if(verbosity)
+         fprintf(stderr, "%s: Total %d triples\n", program,
+                 librdf_model_size(model));
 
        serializer=librdf_new_serializer(world, 
                                         query_graph_serializer_syntax_name, 
@@ -995,7 +1069,8 @@ main(int argc, char *argv[])
        librdf_free_model(tmp_model);
        librdf_free_storage(tmp_storage);
      } else {
-       fprintf(stdout, "%s: Query returned unknown result format\n", program);
+        if(verbosity)
+          fprintf(stderr, "%s: Query returned unknown result format\n", program);
        rc=1;
      }
       
@@ -1049,7 +1124,8 @@ main(int argc, char *argv[])
       
       if((type != CMD_FIND) && (type != CMD_MATCH)) {
         if(!source || !arc || !target) {
-          fprintf(stderr, "%s: cannot have missing triple parts for %s\n", program, cmd);
+          fprintf(stderr,
+                  "%s: cannot have missing triple parts for %s\n", program, cmd);
           librdf_free_statement(partial_statement);
         break;
         }
@@ -1058,11 +1134,17 @@ main(int argc, char *argv[])
     printmatching:
       switch(type) {
         case CMD_CONTAINS:
-          if(librdf_model_contains_statement(model, partial_statement))
-            fprintf(stdout, "%s: the graph contains the triple\n", program);
-        else
-          fprintf(stdout, "%s: the graph does not contain the triple\n", program);
-          break;
+         if(librdf_model_contains_statement(model, partial_statement)) {
+           if(verbosity)
+             fprintf(stderr, "%s: the graph contains the triple\n", program);
+           rc=0;
+         } else {
+           if(verbosity)
+             fprintf(stderr, "%s: the graph does not contain the triple\n",
+                     program);
+           rc=1;
+         }
+         break;
           
         case CMD_FIND:
         case CMD_MATCH:
@@ -1102,14 +1184,16 @@ main(int argc, char *argv[])
           }
 
           if(!stream) {
-            fprintf(stderr, "%s: %s returned no results (NULL stream)\n", program, commands[type].name);
+            fprintf(stderr, "%s: %s returned no results (NULL stream)\n",
+                    program, commands[type].name);
           } else {
             count=0;
             while(!librdf_stream_end(stream)) {
               librdf_statement *statement=librdf_stream_get_object(stream);
-              context_node=(librdf_node*)librdf_stream_get_context(stream);
+              context_node = librdf_stream_get_context2(stream);
               if(!statement) {
-                fprintf(stderr, "%s: librdf_stream_next returned NULL\n", program);
+                fprintf(stderr, "%s: librdf_stream_next returned NULL\n",
+                        program);
                 break;
               }
 
@@ -1155,10 +1239,14 @@ main(int argc, char *argv[])
               fprintf(stderr, "%s: ERROR: Cannot add in context - model does not support contexts\n", program);
             rc=librdf_model_add_statement(model, partial_statement);
           }
-          if(rc)
-            fprintf(stdout, "%s: failed to add triple to the graph\n", program);
-          else
-            fprintf(stdout, "%s: added triple to the graph\n", program);
+          if(verbosity) {
+            if(rc)
+              fprintf(stderr, "%s: Failed to add triple to the graph\n", program);
+            else {
+              if(verbosity)
+                fprintf(stderr, "%s: Added triple to the graph\n", program);
+            }
+          }
           break;
           
         case CMD_REMOVE:
@@ -1173,10 +1261,14 @@ main(int argc, char *argv[])
             librdf_free_node(context_node);
           } else
             rc=librdf_model_remove_statement(model, partial_statement);
-          if(rc)
-            fprintf(stdout, "%s: failed to remove triple from the graph\n", program);
-          else
-            fprintf(stdout, "%s: removed triple from the graph\n", program);
+          if(verbosity) {
+            if(rc)
+              fprintf(stderr, "%s: failed to remove triple from the graph\n", program);
+            else {
+              if(verbosity)
+                fprintf(stderr, "%s: removed triple from the graph\n", program);
+            }
+          }
         break;
         
         default:
@@ -1367,7 +1459,9 @@ main(int argc, char *argv[])
         librdf_iterator_next(iterator);
       }
       librdf_free_iterator(iterator);
-      fprintf(stderr, "%s: matching arcs: %d\n", program, count);
+
+      if(verbosity)
+        fprintf(stderr, "%s: matching arcs: %d\n", program, count);
       
       librdf_free_node(source);
       break;
@@ -1382,10 +1476,13 @@ main(int argc, char *argv[])
       arc=librdf_new_node_from_uri_string(world, (const unsigned char *)argv[1]);
       result=(type == CMD_HAS_ARC_IN) ? librdf_model_has_arc_in(model, arc, source) :
                                         librdf_model_has_arc_out(model, source, arc);
-      if(result)
-        fprintf(stdout, "%s: the graph contains the arc\n", program);
-      else
-        fprintf(stdout, "%s: the graph does not contain the arc\n", program);
+      if(verbosity) {
+        if(result) 
+          fprintf(stderr, "%s: the graph contains the arc\n", program);
+        else
+          fprintf(stderr, "%s: the graph does not contain the arc\n", program);
+      }
+      rc = !result;
       
       librdf_free_node(source);
       librdf_free_node(arc);
@@ -1401,10 +1498,15 @@ main(int argc, char *argv[])
       rc=librdf_model_context_remove_statements(model, context_node);
       librdf_free_node(context_node);
       
-      if(rc)
-        fprintf(stdout, "%s: failed to remove context triples from the graph\n", program);
-      else
-        fprintf(stdout, "%s: removed context triples from the graph\n", program);
+      if(verbosity) {
+        if(rc)
+          fprintf(stderr,
+                  "%s: failed to remove context triples from the graph\n",
+                  program);
+        else
+          fprintf(stderr, "%s: removed context triples from the graph\n",
+                  program);
+      }
       break;
 
 
@@ -1428,21 +1530,22 @@ main(int argc, char *argv[])
         librdf_node_print(node, stdout);
         fputc('\n', stdout);
         
-        librdf_free_node(node);
         count++;
         librdf_iterator_next(iterator);
       }
       librdf_free_iterator(iterator);
-      fprintf(stderr, "%s: contexts: %d\n", program, count);
+
+      if(verbosity)
+        fprintf(stderr, "%s: contexts: %d\n", program, count);
       break;
 
 
     case CMD_SIZE:
       count=librdf_model_size(model);
       if(count >= 0)
-        fprintf(stderr, "%s: graph has %d triples\n", program, count);
+        fprintf(stdout, "%s: graph has %d triples\n", program, count);
       else
-        fprintf(stderr, "%s: graph has unknown number of triples\n", program);
+        fprintf(stdout, "%s: graph has unknown number of triples\n", program);
       break;
 
     default:
@@ -1483,5 +1586,5 @@ main(int argc, char *argv[])
 #endif
 	
   /* keep gcc -Wall happy */
-  return(0);
+  return(rc);
 }
